package javax.swing.text;

import java.awt.Rectangle;
import java.awt.FontMetrics;
import java.awt.Shape;
import sun.swing.SwingUtilities2;
import java.awt.Container;
import javax.swing.JPasswordField;
import java.awt.Graphics;

public class PasswordView extends FieldView
{
    static char[] ONE;
    
    public PasswordView(Element element) {
        super(element);
    }
    
    @Override
    protected int drawUnselectedText(Graphics graphics, int drawEchoCharacter, int n, int n2, int n3) throws BadLocationException {
        Container container = this.getContainer();
        if ((container instanceof JPasswordField)) {
            JPasswordField passwordField = (JPasswordField)container;
            if (!passwordField.echoCharIsSet()) {
                return super.drawUnselectedText(graphics, drawEchoCharacter, n, n2, n3);
            }
            if (passwordField.isEnabled()) {
                graphics.setColor(passwordField.getForeground());
            }
            else {
                graphics.setColor(passwordField.getDisabledTextColor());
            }
            int echoChar = passwordField.getEchoChar();
            for (int n4 = n3 - n2, i = 0; i < n4; ++i) {
                drawEchoCharacter = this.drawEchoCharacter(graphics, drawEchoCharacter, n, (char)echoChar);
            }
        }
        return drawEchoCharacter;
    }
    
    @Override
    protected int drawSelectedText(Graphics graphics, int drawEchoCharacter, int n, int n2, int n3) throws BadLocationException {
        graphics.setColor(this.selected);
        Container container = this.getContainer();
        if ((container instanceof JPasswordField)) {
            JPasswordField passwordField = (JPasswordField)container;
            if (!passwordField.echoCharIsSet()) {
                return super.drawSelectedText(graphics, drawEchoCharacter, n, n2, n3);
            }
            int echoChar = passwordField.getEchoChar();
            for (int n4 = n3 - n2, i = 0; i < n4; ++i) {
                drawEchoCharacter = this.drawEchoCharacter(graphics, drawEchoCharacter, n, (char)echoChar);
            }
        }
        return drawEchoCharacter;
    }
    
    protected int drawEchoCharacter(Graphics g, int x, int y, char ch) {
        PasswordView.ONE[0] = ch;
        SwingUtilities2.drawChars(Utilities.getJComponent(this), g, PasswordView.ONE, 0, 1, x, y);
        return x + g.getFontMetrics().charWidth(ch);
    }
    
    @Override
    public Shape modelToView(int n, Shape shape, Position.Bias bias) throws BadLocationException {
        Container container = this.getContainer();
        if ((container instanceof JPasswordField)) {
            JPasswordField passwordField = (JPasswordField)container;
            if (!passwordField.echoCharIsSet()) {
                return super.modelToView(n, shape, bias);
            }
            int echoChar = passwordField.getEchoChar();
            FontMetrics fontMetrics = passwordField.getFontMetrics(passwordField.getFont());
            Rectangle bounds = this.adjustAllocation(shape).getBounds();
            bounds.x += (n - this.getStartOffset()) * fontMetrics.charWidth((char)echoChar);
            bounds.width = 1;
            return bounds;
        }
        else {
            return null;
        }
    }
    
    @Override
    public int viewToModel(float n, float n2, Shape adjustAllocation, Position.Bias[] array) {
        array[0] = Position.Bias.Forward;
        int n3 = 0;
        Container container = this.getContainer();
        if ((container instanceof JPasswordField)) {
            JPasswordField passwordField = (JPasswordField)container;
            if (!passwordField.echoCharIsSet()) {
                return super.viewToModel(n, n2, adjustAllocation, array);
            }
            int charWidth = passwordField.getFontMetrics(passwordField.getFont()).charWidth(passwordField.getEchoChar());
            adjustAllocation = this.adjustAllocation(adjustAllocation);
            Rectangle rectangle = (adjustAllocation instanceof Rectangle) ? ((Rectangle)adjustAllocation) : adjustAllocation.getBounds();
            n3 = ((charWidth > 0) ? (((int)n - rectangle.x) / charWidth) : Integer.MAX_VALUE);
            if (n3 < 0) {
                n3 = 0;
            }
            else if (n3 > this.getStartOffset() + this.getDocument().getLength()) {
                n3 = this.getDocument().getLength() - this.getStartOffset();
            }
        }
        return this.getStartOffset() + n3;
    }
    
    @Override
    public float getPreferredSpan(int n) {
        switch (n) {
            case 0: {
                Container container = this.getContainer();
                if ((container instanceof JPasswordField)) {
                    JPasswordField passwordField = (JPasswordField)container;
                    if (passwordField.echoCharIsSet()) {
                        int echoChar = passwordField.getEchoChar();
                        FontMetrics fontMetrics = passwordField.getFontMetrics(passwordField.getFont());
                        this.getDocument();
                        return (float)(fontMetrics.charWidth((char)echoChar) * this.getDocument().getLength());
                    }
                    break;
                }
                else {
                    break;
                }
            }
        }
        return super.getPreferredSpan(n);
    }
    
    static {
        PasswordView.ONE = new char[1];
    }
}
